/*
* Copyright (c) 2015 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import UIKit

let ThrowingThreshold: CGFloat = 1000
let ThrowingVelocityPadding: CGFloat = 35

class ViewController: UIViewController {
  @IBOutlet weak var imageView: UIImageView!
  @IBOutlet weak var redSquare: UIView!
  @IBOutlet weak var blueSquare: UIView!

  private var originalBounds = CGRect.zeroRect
  private var originalCenter = CGPoint.zeroPoint

  private var animator: UIDynamicAnimator!
  private var attachmentBehavior: UIAttachmentBehavior!
  private var pushBehavior: UIPushBehavior!
  private var itemBehavior: UIDynamicItemBehavior!

  @IBAction func handleAttachmentGesture(sender: UIPanGestureRecognizer) {
    let location = sender.locationInView(self.view)
    let boxLocation = sender.locationInView(self.imageView)

    switch sender.state {
    case .Began:
      println("Your touch start position is \(location)")
      println("Start location in image is \(boxLocation)")

      animator.removeAllBehaviors()

      let centerOffset = UIOffset(horizontal: boxLocation.x - imageView.bounds.midX, vertical: boxLocation.y - imageView.bounds.midY)
      attachmentBehavior = UIAttachmentBehavior(item: imageView, offsetFromCenter: centerOffset, attachedToAnchor: location)

      redSquare.center = attachmentBehavior.anchorPoint
      blueSquare.center = location

      animator.addBehavior(attachmentBehavior)

    case .Ended:
      println("Your touch end position is \(location)")
      println("End location in image is \(boxLocation)")

      animator.removeAllBehaviors()

      // 1
      let velocity = sender.velocityInView(view)
      let magnitude = sqrt((velocity.x * velocity.x) + (velocity.y * velocity.y))

      if magnitude > ThrowingThreshold {
        // 2
        let pushBehavior = UIPushBehavior(items: [imageView], mode: .Instantaneous)
        pushBehavior.pushDirection = CGVector(dx: velocity.x / 10, dy: velocity.y / 10)
        pushBehavior.magnitude = magnitude / ThrowingVelocityPadding

        self.pushBehavior = pushBehavior
        animator.addBehavior(pushBehavior)

        // 3
        let angle = Int(arc4random_uniform(20)) - 10

        itemBehavior = UIDynamicItemBehavior(items: [imageView])
        itemBehavior.friction = 0.2
        itemBehavior.allowsRotation = true
        itemBehavior.addAngularVelocity(CGFloat(angle), forItem: imageView)
        animator.addBehavior(itemBehavior)

        // 4
        let timeOffset = Int64(0.4 * Double(NSEC_PER_SEC))
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, timeOffset), dispatch_get_main_queue()) {
          self.resetDemo()
        }
      } else {
        resetDemo()
      }

    default:
      attachmentBehavior.anchorPoint = sender.locationInView(view)
      redSquare.center = attachmentBehavior.anchorPoint

      break
    }
  }

  func resetDemo() {
    animator.removeAllBehaviors()

    UIView.animateWithDuration(0.45) {
      self.imageView.bounds = self.originalBounds
      self.imageView.center = self.originalCenter
      self.imageView.transform = CGAffineTransformIdentity
    }
  }

  override func viewDidLoad() {
    super.viewDidLoad()

    animator = UIDynamicAnimator(referenceView: view)
    originalBounds = imageView.bounds
    originalCenter = imageView.center
  }

  override func didReceiveMemoryWarning() {
    super.didReceiveMemoryWarning()
    // Dispose of any resources that can be recreated.
  }


}

